@@ -181,9 +181,8 @@ class InverseTransformTest
     if (absl::StartsWith(test_case, "C/")) {
       memset(base_inverse_transforms_, 0, sizeof(base_inverse_transforms_));
     } else if (absl::StartsWith(test_case, "SSE41/")) {
-      if ((GetCpuInfo() & kSSE4_1) != 0) {
-        InverseTransformInit_SSE4_1();
-      }
+      if ((GetCpuInfo() & kSSE4_1) == 0) GTEST_SKIP() << "No SSE4.1 support!";
+      InverseTransformInit_SSE4_1();
     } else if (absl::StartsWith(test_case, "NEON/")) {
       InverseTransformInit_NEON();
       InverseTransformInit10bpp_NEON();
