@@ -251,6 +251,15 @@ class ObuParser : public Allocable {
   // populated with a valid frame buffer.
   StatusCode ParseOneFrame(RefCountedBufferPtr* current_frame);
 
+  // Get the AV1CodecConfigurationBox as described in
+  // https://aomediacodec.github.io/av1-isobmff/#av1codecconfigurationbox. This
+  // does minimal bitstream parsing to obtain the necessary information to
+  // generate the av1c box. Returns a std::unique_ptr that contains the av1c
+  // data on success, nullptr otherwise. |av1c_size| must not be nullptr and
+  // will contain the size of the buffer pointed to by the std::unique_ptr.
+  static std::unique_ptr<uint8_t[]> GetAV1CodecConfigurationBox(
+      const uint8_t* data, size_t size, size_t* av1c_size);
+
   // Getters. Only valid if ParseOneFrame() completes successfully.
   const Vector<ObuHeader>& obu_headers() const { return obu_headers_; }
   const ObuSequenceHeader& sequence_header() const { return sequence_header_; }
@@ -356,6 +365,13 @@ class ObuParser : public Allocable {
   // otherwise.
   bool EnsureCurrentFrameIsNotNull();
 
+  // Parses the basic bitstream information from the given AV1 stream in |data|.
+  // This is used for generating the AV1CodecConfigurationBox.
+  static StatusCode ParseBasicStreamInfo(const uint8_t* data, size_t size,
+                                         ObuSequenceHeader* sequence_header,
+                                         size_t* sequence_header_offset,
+                                         size_t* sequence_header_size);
+
   // Parser elements.
   std::unique_ptr<RawBitReader> bit_reader_;
   const uint8_t* data_;
